︠69d5cf8f-bb0d-40f2-ba1e-113000c83a0ci︠
%md(hide=true)
<!-- "%md" at the beginning of a cell indicates that cell is in "Markdown" format.-->

## Study of a method to use exponential basis functions to handle positivity issues.

Study how to reconstruct an exponential representation of the solution, given a set of moments of the solution.

[GWH](http://w3.pppl.gov/~hammett) 6/13/2017


︡f8a3565f-39a9-461d-b7f0-6cb6764a9d68︡{"done":true,"md":"<!-- \"%md\" at the beginning of a cell indicates that cell is in \"Markdown\" format.-->\n\n## Study of a method to use exponential basis functions to handle positivity issues.\n\nStudy how to reconstruct an exponential representation of the solution, given a set of moments of the solution.\n\n[GWH](http://w3.pppl.gov/~hammett) 6/13/2017"}
︠d77ff21d-3856-414a-9a10-6ebae5aa41c6s︠
# initializations
%auto
typeset_mode(True) # typeset all output in spreadsheet by default
reset() # reset SageMath to it's initial state (clears all variables)
︡3fc1e732-2d3d-445d-bd8e-00294a6b69ed︡{"done":true}︡
︠ee6ab8e3-0fde-461c-b1c1-f579076c48e2︠
var('f_av fx_av m g A')
︡1ccc2b8c-3c7c-49dd-86fb-af88a134c3c7︡{"html":"<div align='center'>($\\displaystyle f_{\\mathit{av}}$, $\\displaystyle \\mathit{fx}_{\\mathit{av}}$, $\\displaystyle m$, $\\displaystyle g$, $\\displaystyle A$)</div>"}︡{"done":true}︡
︠87cbf54a-84fe-457a-a1c9-fe2e3d51c0b1︠
A=Matrix([[1/2,1/2],[-1/2,1/2]])
︡4fc2374f-5d4d-47fa-9cc5-2d8ec1001229︡{"done":true}︡
︠1ac7fe0d-316b-4282-99fa-455269b05741︠
A
︡15867830-866d-47aa-b90e-d8ead8522105︡{"html":"<div align='center'>$\\displaystyle \\left(\\begin{array}{rr}\n\\frac{1}{2} &amp; \\frac{1}{2} \\\\\n-\\frac{1}{2} &amp; \\frac{1}{2}\n\\end{array}\\right)$</div>"}︡{"done":true}︡
︠479343d9-6305-44c7-8ea1-85fb99a7420f︠
Ainv = A^(-1); Ainv
︡97f92cce-5ab6-4fd2-8d5f-4aae0e3a3110︡{"html":"<div align='center'>$\\displaystyle \\left(\\begin{array}{rr}\n1 &amp; -1 \\\\\n1 &amp; 1\n\\end{array}\\right)$</div>"}︡{"done":true}︡
︠0d8eabc5-9a9a-40e7-9fc0-5ff59a839b33︠
# m is a vector of moments, [<f> = f_0, <xf> = f_1/3]:
m = vector([1.0,0.1])
︡0953862b-f898-4d75-b4c4-d5066bdd0552︡{"done":true}︡
︠e5ca0a05-ce83-4680-8cd0-5cc511641dd1︠
# map the exponential function to the moments, using 3rd order Gauss-Lobatto quadrature:
def F_exp_to_mom(g):
    g0 = g[0]
    g1 = g[1]
    gavg = (g0+g1)/2
    tol = 1.e-6
    g0 = max(g0, tol*gavg)
    g1 = max(g1, tol*gavg)
    return vector( [ g0/6+sqrt(g0*g1)*4/6 + g1/6,  -g0/6+g1/6 ] )
︡0c7e5bcb-9e4b-4191-80f6-f6683a71777b︡{"done":true}︡
︠e46a0f63-3ca9-44d1-b16a-63a530d9c817︠
# map the exponential function to the moments, using 3rd order Gauss-Lobatto quadrature:
def F_exp2_to_mom(g):
    g0 = exp(g[0])
    g1 = exp(g[1])
    return vector( [ g0/6+sqrt(g0*g1)*4/6 + g1/6,  -g0/6+g1/6 ] )
︡1262bcab-fd28-4073-abdd-74b2de429edf︡{"done":true}︡
︠11c3a29b-cf9c-4a69-ad19-e6d70d2a17d4︠
# g=(gL,gR), where gL and gR are the values of the exponential function
# at the left and right boundary.  This then specifies the exponential representation.
g = vector([1.0,1.0])
︡2aa7d78e-e286-4899-8e00-bb94a5d76466︡{"done":true}︡
︠4e3662ba-0688-4ba5-aab6-72bf2ab6ee1a︠
# test the exponential to moments map:
F_exp_to_mom(g)
︡ac48197f-3e88-4093-9835-c08ae5102524︡{"html":"<div align='center'>$\\displaystyle \\left(1.00000000000000,\\,0.000000000000000\\right)$</div>"}︡{"done":true}︡
︠08b41713-3c18-4790-8eb0-8c519b7caf17︠
# The exact inverse function from moments to exponential representation,
# for 3rd order Gauss-Lobattao quadrature.
# (can only be worked out analytically in 1D for 1st and 3rd order Gauss-Lobatto quadrature)
def F_mom_to_exp(m):
    r=m[1]/m[0]
    beta = (2*r+sqrt(3*r^2+1))/(1-r)
    gL = m[0]*6/(1+4*beta+beta^2)
    gR = gL*beta^2
    return vector([gL,gR])
︡f9bfeae3-e045-4491-85b5-4f41cba95389︡{"done":true}︡
︠6c47bf2e-016f-47ef-aa83-f460b7c868cb︠
# Can perhaps rearrange the equations in F_mom_to_exp to be in a simpler or more robust form.  Examples:
%var r
eq1 = F_mom_to_exp(vector([1,r])); eq1
︡3d6480da-813c-441a-b6f3-cff13f4e0660︡{"html":"<div align='center'>$\\displaystyle \\left(\\frac{6}{\\frac{{\\left(2 \\, r + \\sqrt{3 \\, r^{2} + 1}\\right)}^{2}}{{\\left(r - 1\\right)}^{2}} - \\frac{4 \\, {\\left(2 \\, r + \\sqrt{3 \\, r^{2} + 1}\\right)}}{r - 1} + 1},\\,\\frac{6 \\, {\\left(2 \\, r + \\sqrt{3 \\, r^{2} + 1}\\right)}^{2}}{{\\left(r - 1\\right)}^{2} {\\left(\\frac{{\\left(2 \\, r + \\sqrt{3 \\, r^{2} + 1}\\right)}^{2}}{{\\left(r - 1\\right)}^{2}} - \\frac{4 \\, {\\left(2 \\, r + \\sqrt{3 \\, r^{2} + 1}\\right)}}{r - 1} + 1\\right)}}\\right)$</div>"}︡{"done":true}︡
︠8b581638-cb4b-4edf-9519-e8d87b1ac97a︠
eq00 = factor(expand(eq1[0])); eq00
︡382fffa8-ffc7-4b1a-9220-fcdcf97e9bde︡{"html":"<div align='center'>$\\displaystyle \\frac{3 \\, {\\left(r - 1\\right)}^{2}}{3 \\, r + 2 \\, \\sqrt{3 \\, r^{2} + 1} + 1}$</div>"}︡{"done":true}︡
︠1e0503e4-0570-4b9a-838c-81ffa2c4bd48︠
eq10 = factor(expand(eq1[1])); eq10
︡2f9bb8c1-cf2a-4951-abc5-9b59ed5dc864︡{"html":"<div align='center'>$\\displaystyle \\frac{3 \\, {\\left(7 \\, r^{2} + 4 \\, \\sqrt{3 \\, r^{2} + 1} r + 1\\right)}}{3 \\, r + 2 \\, \\sqrt{3 \\, r^{2} + 1} + 1}$</div>"}︡{"done":true}︡
︠3d86b96a-a784-4583-a97e-6a1ad559b45c︠

︡5a900086-9dec-4975-b887-3fc471ddafd8︡{"done":true}︡
︠5db2d926-d077-483d-84ed-59e4a68e4f8b︠
# verify solution:
m = vector([1.0,0.1])
F_mom_to_exp(m)
F_exp_to_mom(F_mom_to_exp(m))
︡a090f327-0465-48fa-a1fd-30f7a0d198aa︡{"html":"<div align='center'>$\\displaystyle \\left(0.729778313018444,\\,1.32977831301844\\right)$</div>"}︡{"html":"<div align='center'>$\\displaystyle \\left(1.00000000000000,\\,0.100000000000000\\right)$</div>"}︡{"done":true}︡
︠1afef420-15dc-493c-b1a6-7cf24123d584︠
m = vector([1.0,0.99999])
F_mom_to_exp(m)
F_exp_to_mom(F_mom_to_exp(m))
︡0e9f1793-4ddc-4c0a-8fea-c86efbec5a86︡{"html":"<div align='center'>$\\displaystyle \\left(3.75002812515923 \\times 10^{-11},\\,5.99994000003750\\right)$</div>"}︡{"html":"<div align='center'>$\\displaystyle \\left(1.00281889884175,\\,0.999989500011250\\right)$</div>"}︡{"done":true}︡
︠950af77a-98c6-448f-8881-f8aeb9e03565︠
# A very approximate function from moments to exponential representation,
# from the 1st order Gauss-Lobattao quadrature.
#
def F_mom_to_exp_GL1(m):
    return vector([m[0]-m[1],m[0]+m[1]])
︡f4bab509-8dee-417e-8fb8-1a2a7f63906d︡{"done":true}︡
︠a3c71912-cdec-4b9e-9bed-0dc3e196982f︠
def F_x3(x,m):
    f0 = m[0]
    xavg = m[1]
    # the normalization in the denominator is to ensure the average f
    # is correct when evaluated with 3rd order Gauss-Lobatto quadrature
    #return f0*(1+xavg*x)^3/((1/6)*(1-xavg)^3 + 4/6 + (1/6)*(1+xavg)^3)
    # However, that doesn't give enough enhancement of f at the boundary,
    # so try the simpler expression:
    return f0*(1+xavg*x)^3
︡788da8e8-8643-4617-bdb0-a1ed82a6d377︡{"done":true}︡
︠bd25c6b8-3e02-46a8-9cb0-703c7107d737︠
def F_x3rt(m):
    f0 = m[0]
    xavg = m[1]
    return f0*(1+xavg)^3 / ( (1/6)*(1-xavg)^3 + (4/6)*(1-xavg)^1.5 * (1+xavg)^1.5 + (1/6)*(1+xavg)^3 )
︡408d55b2-6a18-4fd5-9c27-9ecc49f7634e︡{"done":true}︡
︠f74b6439-3444-41c4-b6b6-efb1d6c43ed2︠
def F_mom_to_exp_x3(m):
    return F_x3(1,m)
︡b4d4b240-f744-4b0c-9d7c-637226e62b1c︡{"done":true}︡
︠fff56c7a-8e58-4604-929c-fa6fd6c6d2f1︠
# Make a plot
eq1 = F_mom_to_exp(vector([1,x]))[1]
eq2 = F_mom_to_exp_GL1(vector([1,x]))[1]
eq_x3 = F_x3(1,vector([1,x]))
eq_x3rt = F_x3rt(vector([1,x]))
P = plot(eq1,legend_label='exponential g @x=1')
P = P + plot(1+3*x,color="green",legend_label='linear g @x=1')
P = P + plot(eq2,color="gray",legend_label='rough g@x=1')
P = P + plot(eq_x3, color='red', legend_label='(1+<x>)^3')
P = P + plot(eq_x3rt, color='orange', legend_label='(1+<x>)^3 w exp interp')
P.axes_labels(['<x> = <xf>/<f>','g_R'])
P.show()
︡2cd4c9cf-4bf2-4893-86d4-24d02bb4268c︡{"file":{"filename":"/projects/b0fff224-ee44-4488-bd52-7a9b55bcf3db/.sage/temp/compute4-us/17781/tmp__SLMGA.svg","show":true,"text":null,"uuid":"447420ec-3851-451c-8d62-a970b54969ea"},"once":false}︡{"done":true}︡
︠9ccf6913-c751-4e93-9691-ebe964f28b7ei︠
%md(hide=true)
The above plot demonstrates that the approximation for g_R($\langle x \rangle$) base on 1st order Gauss-Lobatto quadrature is *not* 1st order accurate.  This is because calculating $\langle x f(x) \rangle$ involves $\langle x^2 \rangle$ terms, which requires more than 1st order accuracy to be accurate.  At least it is a postive definite approximation.
︡f5edbd87-76f8-4fa8-9533-bc0c1bc4cef0︡{"done":true,"md":"The above plot demonstrates that the approximation for g_R($\\langle x \\rangle$) base on 1st order Gauss-Lobatto quadrature is *not* 1st order accurate.  This is because calculating $\\langle x f(x) \\rangle$ involves $\\langle x^2 \\rangle$ terms, which requires more than 1st order accuracy to be accurate.  At least it is a postive definite approximation."}
︠0142a149-57bc-4ccd-adb3-96d05eaa7d49︠
# choose a set of moments:
m = vector([1.0,0.7])

g_exact = F_mom_to_exp(m)
print "g_exact =", g_exact

# iniia guess
g = Ainv * m
print raw g_int =", g

#resal solutio o xactly give the corect ensty:
g= g/F_xp_o_momg)[]

# Note: I wonder if rescaling the solution alters the 
# convergence rate of Newton in some way.  I think
# is should be okay, but perhaps think a little more.

%var err_est, s, frac
rel_frac = 0.5

print "g_init  =", g
︡e35da177-98ab-44c5-8c82-4839d889e092︡{"stdout":"g_exact = (0.0432467291003422, 4.24324672910034)\n"}︡{"stdout":"raw g_init = (0.300000000000000, 1.70000000000000)\n"}︡{"stdout":"g_init  = (0.370631843401647, 2.10024711260933)\n"}︡{"done":true}︡
︠caec2afd-0b19-40ae-86ba-14b17f3f245c︠
# Repeatedly run this cell to do iterations of a preconditioned iterative solver:

g_old = g
r = F_exp_to_mom(g) - m
print "previous g=", g
print "residual =", r, "|res| =",abs(r)
print "error =", g-g_exact

err_est = Ainv*r

print "err_est =", err_est

# Do enough back tracking to ensure that no element of g
# drops by more than "frac" in a single iteration
# (this keeps g from going negative):
s = 1
if (err_est[0] > rel_frac*g[0]): s = rel_frac*g[0]/err_est[0]
if (err_est[1] > rel_frac*g[1]): s = min(s,rel_frac*g[1]/err_est[1])

g = g - s*err_est

# The positivity backtracking is not enough, also have to check the error
# to see if we have to backtrack more:
#

res_new = abs(F_exp_to_mom(g) - m)

if(res_new > abs(r)): 
    print "s, res, res_new=", s, res_new, abs(r)
    s = s/2

g = g - s*err_est
res_new = abs(F_exp_to_mom(g) - m)


if(res_new > abs(r)): 
    print "s, res, res_new=", s, abs(r), res_new
    s = s/2

g = g - s*err_est
res_new = abs(F_exp_to_mom(g) - m)


# older method of limiting:

# print "raw g = ", g
#epsilon=1.e-6
#g[0] = max(epsilon, g[0])
#g[1] = max(epsilon, g[1])

print "s =", s
print "g before rescaling:", g
# rescale solution to exactly correct density:
g = g / ( F_exp_to_mom(g)[0] )
print "g                 :", g, "res = ", F_exp_to_mom(g) - m
︡1461cb2a-7461-4a5d-b199-1028032f8100︡{"stdout":"previous g= (0.370631843401647, 2.10024711260933)\n"}︡{"stdout":"residual = (2.22044604925031e-16, -0.411730788465385) |res| = 0.411730788465385\n"}︡{"stdout":"error = (0.327385114301305, -2.14299961649101)\n"}︡{"stdout":"err_est = (0.411730788465386, -0.411730788465385)\n"}︡{"stdout":"s, res, res_new= 0.450090027009003 0.411730788465385 0.653946807970084\n"}︡{"stdout":"s = 0.225045013504502\n"}︡{"stdout":"g before rescaling: (-0.0926579608504118, 2.56353691686139)\n"}︡{"stdout":"g                 : (-0.216266827161853, 5.98338221814460) res =  (2.22044604925031e-16, 0.297229889097817)\n"}︡{"done":true}︡
︠f5daf6a6-249a-4748-9635-9c4a371ae32a︠
# The above fails to converge for a slope of 0.7.  Might be able to improve the backtracking or the estimated Jacobian.
︡b26978ae-f05a-4266-b318-0722c9a5f3e4︡
︠4e0d898c-6766-47af-a2b7-69c1cf4f0e40︠
g_old
"old res", abs(F_exp_to_mom(g_old) - m)
g_new = g_old-0.5*err_est
g_new = g_new / ( F_exp_to_mom(g_new)[0] )
abs(F_exp_to_mom(g_new) - m)
︡e5d2868b-a791-41b1-909f-d8c62846a2c2︡{"html":"<div align='center'>$\\displaystyle \\left(0.0405996888408913,\\,4.29003378752084\\right)$</div>"}︡{"html":"<div align='center'>(old res, $\\displaystyle 0.00823901644665892$)</div>"}︡{"html":"<div align='center'>$\\displaystyle 0.00264206777375164$</div>"}︡{"done":true}︡
︠f22cac0f-29e6-4d5f-9129-1edb6b332aa4︠

︡9e091bfe-8b65-4f76-9064-5fc9fca43c69︡
︠e62acde2-5271-4ac6-8f49-0c9ed97729eci︠
%md(hide=true)
#### try SciPy numerical nonlinear solvers
︡682cb77f-6f63-4681-92f6-e9e20aab6022︡{"done":true,"md":"#### try SciPy numerical nonlinear solvers"}
︠6dc62b3c-2aea-4599-be62-3028abc8f7e4︠
def fun(g):
    return F_exp2_to_mom(g)-m
︡8905fa09-2854-4b69-b8bf-79bb6a78e825︡{"done":true}︡
︠e39d472d-c82f-4a2b-aa70-f9cb99dbe863︠
from scipy import optimize
︡544a38fc-5e0f-4a79-a49e-f21ae449bc0b︡{"done":true}︡
︠0c98bbe7-4933-44e3-a878-1c704376db41︠
m = vector([1.0,0.99])
g_exact = F_mom_to_exp(m)
g_init = vector([log(m[0]-m[1]), log(m[0]+m[1])])
# sol = optimize.root(fun, g_init, jac=false, method='hybr', tol=1.e-1)
sol = optimize.root(fun, g_init, jac=false, method='hybr', options={'xtol':1.e-3})
︡aea48567-53f6-44e1-9282-0e245c2a47fb︡{"done":true}︡
︠08ad806e-efee-43d3-83fd-db3b22cf5014︠
sol
︡10c8eb47-5b9c-423e-968d-429036a7c6ff︡{"stdout":"    fjac: array([[ -9.99999851e-01,   5.45450257e-04],\n       [ -5.45450257e-04,  -9.99999851e-01]])\n     fun: array([  2.87526888e-07,   6.24956084e-08])\n message: 'The solution converged.'\n    nfev: 14\n     qtf: array([ -2.54857935e-05,  -2.58130665e-06])\n       r: array([-0.00508839, -0.96073911, -1.00869966])\n  status: 1\n success: True\n       x: array([-10.18360123,   1.78171556])\n"}︡{"done":true}︡
︠753e2881-97f3-472d-b7f0-054b4dd9dbb6︠
# number of iterations for various values of the slope (<x>), with GL1 initial guess:
# tol=1.e-3 (residual --> 1.e-17)
#
# slope = 0.1,    nfev = 7
# slope = 0.8,    nfev = 12 res ~ 1.e-6
# slope = 0.9999, nfev = 17 res ~ 1.e-7
# slope = 0.9999, nfev = 9 if tol = 0.1, then res jumps up to ~ 1.e-3

# number of iterations for various values of the slope (<x>), with GL1 initial guess:
# tol=default (residual --> 1.e-17)
#
# slope = 0.1,    nfev = 10
# slope = 0.2,    nfev = 11
# slope = 0.7,    nfev = 15
# slope = 0.8,    nfev = 15
# slope = 0.9,    nfev = 16
# slope = 0.99,   nfev = 17
# slope = 0.999,  nfev = 18
# slope = 0.9999, nfev = 20

# number of iterations for various values of the slope (<x>), simplest initial guess
# g_init = (1,1)
#
# slope = 0.1,    nfev = 10
# slope = 0.7,    nfev = 18
# slope = 0.8,    nfev = 18
# slope = 0.9,    nfev = 23
# slope = 0.99,   nfev = 31
# slope = 0.999,  nfev = 33
# slope = 0.9999, nfev = 39
︡9ce99d8f-aded-4438-81a4-36a4d6541388︡{"done":true}︡
︠59434abf-2a54-459c-9f76-0c01ce14e2cc︠
# Could use the the Jacobian matrix, which isn't that hard to work out:
moms = F_exp_to_mom(vector([g0,g1])); moms
︡3bf4d0d0-2565-492b-893d-c123b0c35cee︡{"html":"<div align='center'>$\\displaystyle \\left(\\frac{1}{6} \\, g_{0} + \\frac{1}{6} \\, g_{1} + \\frac{2}{3} \\, \\sqrt{g_{0} g_{1}},\\,-\\frac{1}{6} \\, g_{0} + \\frac{1}{6} \\, g_{1}\\right)$</div>"}︡{"done":true}︡
︠3206d969-230e-43a6-824c-bf411b8300f1︠
diff(moms[0],g0)
diff(moms[0],g1)
diff(moms[1],g0)
diff(moms[1],g1)
︡ab7deeb0-bb17-43d9-a721-d2f4f73b177e︡{"html":"<div align='center'>$\\displaystyle \\frac{g_{1}}{3 \\, \\sqrt{g_{0} g_{1}}} + \\frac{1}{6}$</div>"}︡{"html":"<div align='center'>$\\displaystyle \\frac{g_{0}}{3 \\, \\sqrt{g_{0} g_{1}}} + \\frac{1}{6}$</div>"}︡{"html":"<div align='center'>$\\displaystyle -\\frac{1}{6}$</div>"}︡{"html":"<div align='center'>$\\displaystyle \\frac{1}{6}$</div>"}︡{"done":true}︡
︠5acedf8c-8a61-46d7-ac18-7aa2829798ca︠
var('r, m0, m1, g0, g1, beta')
︡e3b86047-6fbf-4e22-be09-9fb4d485262c︡{"html":"<div align='center'>($\\displaystyle r$, $\\displaystyle m_{0}$, $\\displaystyle m_{1}$, $\\displaystyle g_{0}$, $\\displaystyle g_{1}$, $\\displaystyle \\beta$)</div>"}︡{"done":true}︡
︠3c19f30b-de4b-472d-9181-200fc745a28b︠
var('feq(x,y)')
︡89d9bb81-2be2-47bb-a293-f362b2ac2041︡{"stderr":"Error in lines 1-1\nTraceback (most recent call last):\n  File \"/projects/sage/sage-7.5/local/lib/python2.7/site-packages/smc_sagews/sage_server.py\", line 995, in execute\n    exec compile(block+'\\n', '', 'single') in namespace, locals\n  File \"\", line 1, in <module>\n  File \"/projects/sage/sage-7.5/local/lib/python2.7/site-packages/smc_sagews/sage_salvus.py\", line 3034, in var\n    return var0(*args, **kwds)\n  File \"/projects/sage/sage-7.5/local/lib/python2.7/site-packages/smc_sagews/sage_salvus.py\", line 2996, in var0\n    v = sage.all.SR.var(name, **kwds)\n  File \"sage/symbolic/ring.pyx\", line 776, in sage.symbolic.ring.SymbolicRing.var (/projects/sage/sage-7.5/src/build/cythonized/sage/symbolic/ring.cpp:9863)\n    raise ValueError('The name \"'+s+'\" is not a valid Python identifier.')\nValueError: The name \"feq(x\" is not a valid Python identifier.\n"}︡{"done":true}︡
︠6702b10a-d30a-46d7-9dbb-1c085491eed9︠
def int2d_GL3(n,m,f):
    '''Function to calculate <x^n*y^m*f(x,y)> with 3rd order Gauss-Lobatto quadrature.
    '''
    # (Note, this works because 0^0 is defined as 1.)
    int = (  (1/6)*((-1)^m*(-1)^n*f(-1,-1)/6 + (0)^m*(-1)^n*f(0,-1)*4/6 + (1)^m*(-1)^n*f(1,-1)/6)
           + (4/6)*((-1)^m*(1)^n*f(-1,0)/6  + (-1)^m*(1)^n*f(0,0)*4/6  + (-1)^m*(1)^n*f(1,0)/6)
           + (1/6)*((-1)^m*(0)^n  *f(-1,0)/6  + (0)^m *(0)^n*f(0,0)*4/6  + (1)^m *(0)^n*f(1,0)/6))
    return int
︡b6412c62-fe7b-46d4-9ca8-774dcfeda4e6︡{"done":true}︡
︠a2d6229b-5cf6-406c-97c1-33226a22c5e8︠
_=var("g00 g10 g01 g11")fexp(x,y) = g00^((1-x)/2*(1-y)/2) * g01^((1+x)/4*(1-y)/2)  *
 f10^((1+x)/2*(1+y)/2) * g11^((1-x)/2+*(1+y)/2)
︡b4304f6d-c05b-46cb-a6c0-a10c30e85ae3︡
︠8cc0c57a-5379-4277-95cc-c3f54dba27b0︠
# verify this givs all 9 values for Gauss-Lobatto integration:
fexp(-1,-1)
fexp(0,-1)
fexp(1,-1)
fexp(-1,1)
fexp(0,0)
fexp(1,0)
fexp(-1,1)
fexp(0,1)
fexp(1,1)

︡095b637f-e423-4f7d-9e9e-08aa0f4d95f4︡{"stdout":"g00^(1/4*(x - 1)*(y - 1))*g01^(-1/4*(x - 1)*(y + 1))*g10^(-1/4*(x + 1)*(y - 1))*g11^(1/4*(x + 1)*(y + 1))9291d4d6-f7ca-4b62-99b0-8c95ebae902f︡{"stdout":"g00\n"}︡{"stdout":"sqrt(g00)*sqrt(gg10)\n"}︡{"stdout":"gg10\n"}︡{"stdout":"sqrt(g00)*sqrt(g01)\n"}︡{"stdout":"g00^(1/4)*g01^(1/4)*g10^(1/4)*g11^(1/4)\n"}︡{"stdout":"sqrt(g10)*sqrt(g11)\n"}︡{"stdout":"f01\n"}︡{"stdout":"sqrt(f01)*sqrt(f11)\n"}︡{"stdout":"f11\n"}b-f9b737619bbe︡{"html":"<div align='center'>($\\displaystyle f_{00}$, $\\displaystyle f_{10}$, $\\displaystyle f_{01}$, $\\displaystyle f_{11}$)</div>"}
︠9f70b1fd-6f62-4594-a6c3-b787fec28434︠
fexp()
︡3ac67764-134a-45ae-a2ee-ef005d39549d︡
︠29ffae1a-5996-448e-8256-70d0216f1c45︠
var('f00 f10 f01 f11')
︡bbcdb2c7-5712-4135-a152-d1716525d464︡
︠f633b14c-ea2c-44d5-9b25-ab06e5ea5797︠
eq22 = f01 == int2d_GL3(0,1,fexp)
︡f6438294-662c-48b4-ac0f-ffe90b647727︡{"done":true}︡
︠d6cf5adb-4de2-46b3-a4b6-91463c351353︠
eq23 = f11 == int2d_GL3(1,1,fexp)
︡4c8eb768-bdb6-4a7c-83b3-3fac0f8c6ca7︡{"done":true}︡
︠3188f28e-ea72-4c3f-8191-457f9e7549a8︠
eq20
eq21
eq22
eq23
︡c8ac7550-ac3e-4f6f-ba62-8d6dc8f5fdbc︡{"stdout":"f00 == 1/36*g00 + 5/36*sqrt(g00)*sqrt(g01) + 1/9*sqrt(g00)*sqrt(g10) + 1/36*g10 + 5/9*g00^(1/4)*g01^(1/4)*g10^(1/4)*g11^(1/4) + 5/36*sqrt(g10)*sqrt(g11)\n"}︡{"stdout":"f10 == -1/36*g00 + 1/36*sqrt(g00)*sqrt(g01) - 1/9*sqrt(g00)*sqrt(g10) - 1/36*g10 + 1/9*g00^(1/4)*g01^(1/4)*g10^(1/4)*g11^(1/4) + 1/36*sqrt(g10)*sqrt(g11)\n"}︡{"stdout":"f01 == -1/36*g00 - 5/36*sqrt(g00)*sqrt(g01) + 1/36*g10 - 1/9*g00^(1/4)*g01^(1/4)*g10^(1/4)*g11^(1/4) + 1/12*sqrt(g10)*sqrt(g11)\n"}︡{"stdout":"f11 == 1/36*g00 - 1/36*sqrt(g00)*sqrt(g01) - 1/36*g10 - 1/9*g00^(1/4)*g01^(1/4)*g10^(1/4)*g11^(1/4) - 1/36*sqrt(g10)*sqrt(g11)\n"}︡{"done":true}︡
︠503a6876-ec0d-48b7-836e-cab28b4a9060︠
diff(eq20,g00) ; diff(eq20, g10) ; diff(eq20,g01) ; diff(eq20, g11)
︡dd2a6ea1-b982-4539-9513-eb65b37a55a9︡{"stdout":"0 == 5/72*sqrt(g01)/sqrt(g00) + 1/18*sqrt(g10)/sqrt(g00) + 5/36*g01^(1/4)*g10^(1/4)*g11^(1/4)/g00^(3/4) + 1/36\n0 == 1/18*sqrt(g00)/sqrt(g10) + 5/36*g00^(1/4)*g01^(1/4)*g11^(1/4)/g10^(3/4) + 5/72*sqrt(g11)/sqrt(g10) + 1/36\n0 == 5/72*sqrt(g00)/sqrt(g01) + 5/36*g00^(1/4)*g10^(1/4)*g11^(1/4)/g01^(3/4)\n0 == 5/36*g00^(1/4)*g01^(1/4)*g10^(1/4)/g11^(3/4) + 5/72*sqrt(g10)/sqrt(g11)\n"}︡{"done":true}︡
︠0c5fe060-214f-487b-b941-b4cf63b3aa59︠
eq20 = f00 == int2d_GL3(0,0,fexp)
︡d2dfb146-fd36-4882-8cc6-797df060aace︡︡{"done":true}
︠97ff85ad-183e-4772-a102-1fd734dfa6a4︠
eq21 = f10 == int2d_GL3(1,0,fexp)
︡06913187-016f-4568-a30c-40a0bf2c5790︡{"done":true}︡
︠b0dc0561-5272-4426-9923-f51ceeb87091i︠
︡c47854dd-c799-4190-8a54-862b9d6a4c9d︡
︠ed0fc4d1-df8e-4c53-a9c6-dcc99e79fc9b︠
eq
fL^((1-x)/2*(1-y)/2)
︡1b173c68-0042-4a2a-8f36-5aa02feeb7d6︡{"stdout":"fL^(1/4*(x - 1)*(y - 1))\n"}︡{"done":true}︡
︠d01fcd14-b82e-4c51-b237-b1486a505433︠
# 1st order Gauss-Lobattao moments:
︡bed723ea-44e7-4801-8b21-d78f57509021︡{"done":true}︡
︠efa16da0-616d-4350-8033-39f53838e0d4︠
eq10 = f001 == (-g00-g10+g01+g11)/4
eq11 = f10 == (-g00+g10-g01+g11)/4
eq13 = f11 == (g00-g10-g01+g11)/4
eq10 = f00 == (g00+g10+g01+g11)/4
︡99872f43-afe3-4981-a17f-3ccaaeb951a9︡{"done":true}︡
︠9c83849c-0114-4682-a2e7-08d62493cbb9︠
corner_GL1 = solve([eq10,eq11,eq12,eq13], g00, g10, g01, g11)
︡a3e0dd3d-20bb-4e62-8ec3-8efc8945b855︡{"done":true}︡
︠dfdae74e-3cdf-4395-ad36-80b5861f7e73︠
(16+2*4*4+16+2*(-8)+2*4*(-2)+4)
︡71109ccd-f992-4201-b94e-83667cd7a081︡{"html":"<div align='center'>$\\displaystyle 36$</div>"}︡{"done":true}︡
︠4f356f72-e381-4807-8dd5-239dc8bca0aa︠
corner_GL1
︡e62a60ee-01be-4927-814a-7b768bed66f6︡{"html":"<div align='center'>[[$\\displaystyle g_{00} = f_{00} - f_{01} - f_{10} + f_{11}$, $\\displaystyle g_{10} = f_{00} - f_{01} + f_{10} - f_{11}$, $\\displaystyle g_{01} = f_{00} + f_{01} - f_{10} - f_{11}$, $\\displaystyle g_{11} = f_{00} + f_{01} + f_{10} + f_{11}$]]</div>"}︡{"done":true}︡
︠a8525465-cd74-4042-a140-9bb20ad1d9f4︠
corner_GL1[0][j].substitute(f00=1, 10=-1, f01=-1, f10=0, f11=0)
︡67707b3b-e662-4333-a135-aa2231c47f29︡{"html":"<div align='center'>$\\displaystyle g_{10} = 0$</div>"}︡{"done":true}
︠4e219144-cdda-4fa5-9132-1d87664c2220︠
for j in range(4):
    corner_GL1[0][1].substitute(f00=1, f0f01=1, f11=1)
︡491b24c1-14b9-4542-bdc4-2f81ef1bc224︡{"html":"<div align='center'>$\\displaystyle g_{00} = 4$</div>"}︡{"html":"<div align='center'>$\\displaystyle g_{10} = 0$</div>"}︡{"html":"<div align='center'>$\\displaystyle g_{01} = 0$</div>"}︡{"html":"<div align='center'>$\\displaystyle g_{11} = 4$</div>"}︡{"done":true}︡
︠2656dcce-f714-4a69-b96e-2308abd10b30i︠
%md(hide=true)
Test out iteration with limiters
︡834c9984-7d46-4940-b7db-60eb0f11f989︡{"done":true,"md":"Test out iteration with limiters"}
︠a5740173-3b71-4448-ae13-35cd252493b8︠
%var xn, xnp1, lam

lam = 0.2
xn = 0
︡110bc599-5ff9-4da8-a4c4-f452414e1921︡{"done":true}︡
︠7c60adcf-686d-4795-abb5-e305e604a3cc︠
xnp1 = (1 - (1 - xn - 2 * lam) -lam*(1+xn)^3 ) / (1 - lam*(1+xn)^3)
xn = xnp1; x
︡22ee5230-4d38-40e7-bc3d-f91542b080d1︡{"html":"<div align='center'>$\\displaystyle x$</div>"}︡{"done":true}︡
︠10e7f09b-f1df-4f51-9de3-364c6e8b9019i︠
%md(hide=true)
Here are the full iteration equations, for 1D passive advection, for a cell that has no flux into it:
︡d63e80df-eb1d-45ad-889b-314786187ef0︡{"done":true,"md":"Here are the full iteration equations, for 1D passive advection, for a cell that has no flux into it:"}
︠b3d55f66-1655-404c-832f-2850ed2ad722s︠
reset() # reset all variables
︡54afc0de-a577-4bb8-a81d-a15d00ff17da︡{"done":true}︡
︠79ecea21-efe0-46ca-a27d-bb661f926425s︠
%var f_0_n, F_R, lam, f_0_np1
eq30 = f_0_np1 == f_0_n - lam * F_R
eq30
︡60d8c078-88c2-4bd3-80e9-b56ea05f9ffd︡{"html":"<div align='center'>$\\displaystyle f_{0_{\\mathit{np}_{1}}} = -F_{R} \\mathit{lam} + f_{0_{n}}$</div>"}︡{"done":true}︡
︠faafc0de-76e3-4ac7-85bc-052a5821ba1es︠
%var f_1_np1, f_1_n, lam, beta
eq31 = f_1_np1 == f_1_n - 3*lam*F_R + 6*lam*beta*f_0_n
eq31
︡3fd27bfd-6da6-4fa0-a480-8059a4db0767︡{"html":"<div align='center'>$\\displaystyle f_{1_{\\mathit{np}_{1}}} = 6 \\, \\beta f_{0_{n}} \\mathit{lam} - 3 \\, F_{R} \\mathit{lam} + f_{1_{n}}$</div>"}︡{"done":true}︡
︠30cdacda-6f18-44e1-bc69-57be17b7c7ffs︠
%var r_np1, r_n, R_R
eq32 = r_np1 == eq31.rhs() / eq30.rhs()
eq32
︡91b287c3-71b3-416e-91c9-06fee82db930︡{"html":"<div align='center'>$\\displaystyle r_{\\mathit{np}_{1}} = -\\frac{6 \\, \\beta f_{0_{n}} \\mathit{lam} - 3 \\, F_{R} \\mathit{lam} + f_{1_{n}}}{F_{R} \\mathit{lam} - f_{0_{n}}}$</div>"}︡{"done":true}︡
︠698ad87f-9b05-4572-8d18-85c68082dee1s︠
%var theta, x_n, x_np1
eq33 = eq32.substitute(f_1_n=f_0_n*3*x_n, F_R = theta*f_0_n*(1+3*x_n))
eq33
︡05943124-8ee9-49c6-a3cb-72a56a2eee5d︡{"html":"<div align='center'>$\\displaystyle r_{\\mathit{np}_{1}} = \\frac{3 \\, {\\left(f_{0_{n}} \\mathit{lam} \\theta {\\left(3 \\, x_{n} + 1\\right)} - 2 \\, \\beta f_{0_{n}} \\mathit{lam} - f_{0_{n}} x_{n}\\right)}}{f_{0_{n}} \\mathit{lam} \\theta {\\left(3 \\, x_{n} + 1\\right)} - f_{0_{n}}}$</div>"}︡{"done":true}︡
︠ed6a2e65-6c70-4459-833b-c0738e2da484s︠
eq34 = x_np1 == (eq33.rhs())/3
eq34
︡8b23a7a9-27a9-4bdf-adaa-331753fcf8d5︡{"html":"<div align='center'>$\\displaystyle x_{\\mathit{np}_{1}} = \\frac{f_{0_{n}} \\mathit{lam} \\theta {\\left(3 \\, x_{n} + 1\\right)} - 2 \\, \\beta f_{0_{n}} \\mathit{lam} - f_{0_{n}} x_{n}}{f_{0_{n}} \\mathit{lam} \\theta {\\left(3 \\, x_{n} + 1\\right)} - f_{0_{n}}}$</div>"}︡{"done":true}︡
︠e279dd91-1762-4bc3-8879-88c4e61bc4cfs︠
eq34 = eq34.lhs() == factor(eq34.rhs())
eq34
︡aa50a3b6-ce82-48a2-b42f-c50ff8b72d6a︡{"html":"<div align='center'>$\\displaystyle x_{\\mathit{np}_{1}} = \\frac{3 \\, \\mathit{lam} \\theta x_{n} - 2 \\, \\beta \\mathit{lam} + \\mathit{lam} \\theta - x_{n}}{3 \\, \\mathit{lam} \\theta x_{n} + \\mathit{lam} \\theta - 1}$</div>"}︡{"done":true}︡
︠80749615-a4f1-460e-a9e4-fdb7fae803das︠
eq35 = eq34.substitute(x_np1 = x_n)
eq35
︡0ea38b87-8591-4935-9192-0666d19616f2︡{"html":"<div align='center'>$\\displaystyle x_{n} = \\frac{3 \\, \\mathit{lam} \\theta x_{n} - 2 \\, \\beta \\mathit{lam} + \\mathit{lam} \\theta - x_{n}}{3 \\, \\mathit{lam} \\theta x_{n} + \\mathit{lam} \\theta - 1}$</div>"}︡{"done":true}︡
︠1b5b7b07-2e3d-474f-afd6-7f4092324b4bs︠
# search for fixed points:
eq36 = solve(eq35, x_n)
eq36
︡7b7058e9-58b1-4c49-8018-dc039a7c6294︡{"html":"<div align='center'>[$\\displaystyle x_{n} = \\frac{\\theta - \\sqrt{-6 \\, \\beta \\theta + 4 \\, \\theta^{2}}}{3 \\, \\theta}$, $\\displaystyle x_{n} = \\frac{\\theta + \\sqrt{-6 \\, \\beta \\theta + 4 \\, \\theta^{2}}}{3 \\, \\theta}$]</div>"}︡{"done":true}︡
︠b2e0b2de-1d5a-4c9e-a89e-387d299d2578s︠
eq37 = eq36[1].rhs()
eq37
︡427ab0ca-8b82-4095-8379-ae8e7895c4b5︡{"html":"<div align='center'>$\\displaystyle \\frac{\\theta + \\sqrt{-6 \\, \\beta \\theta + 4 \\, \\theta^{2}}}{3 \\, \\theta}$</div>"}︡{"done":true}︡
︠06c2693a-6670-4372-8a21-983deb6d0b53s︠
eq37 = eq37.substitute(beta=1)
eq37
︡ae5e08c8-274b-45c3-a4b7-9f90981fe9a5︡{"html":"<div align='center'>$\\displaystyle \\frac{\\theta + \\sqrt{4 \\, \\theta^{2} - 6 \\, \\theta}}{3 \\, \\theta}$</div>"}︡{"done":true}︡
︠7dcdd784-eb1d-4bc8-b93f-d702832078b2s︠
P = plot(eq37,(theta,1.5,5))
P.axes_labels(["theta","x_fixed"])
show(P)
︡49b980b6-7fa2-484e-aedc-d7552dc151f2︡{"file":{"filename":"/projects/b0fff224-ee44-4488-bd52-7a9b55bcf3db/.sage/temp/compute4-us/31153/tmp_hTR0kV.svg","show":true,"text":null,"uuid":"59b5931e-6fea-4c88-9afd-14b6778899d5"},"once":false}︡{"done":true}︡
︠2d9aa703-cb1e-475f-ab93-2cd78390e14es︠
# The above is a plot of the fixed point as a function of theta, but it doesn't really make much sense, because theta should be a function of x and needs to reduce to theta=1 when |x|<<1.
︡cf5cb9f0-7519-4f6b-bfc4-57ae7875c2d1︡{"done":true}︡
︠4f69d72a-903e-4088-b688-f1442fb1c091s︠
eq32
︡3e752062-f3c1-488d-9553-c6e95172cda1︡{"html":"<div align='center'>$\\displaystyle r_{\\mathit{np}_{1}} = -\\frac{6 \\, \\beta f_{0_{n}} \\mathit{lam} - 3 \\, F_{R} \\mathit{lam} + f_{1_{n}}}{F_{R} \\mathit{lam} - f_{0_{n}}}$</div>"}︡{"done":true}︡
︠c683b845-c70c-4a11-9379-d83aee81efb3s︠
%var F_R_rel
eq40 = factor(eq32.substitute(f_1_n=f_0_n*3*x_n, F_R = f_0_n*F_R_rel)); eq40
︡335652a3-bd4f-4364-9fee-9b3d100576c4︡{"html":"<div align='center'>$\\displaystyle r_{\\mathit{np}_{1}} = \\frac{3 \\, {\\left(F_{R_{\\mathit{rel}}} \\mathit{lam} - 2 \\, \\beta \\mathit{lam} - x_{n}\\right)}}{F_{R_{\\mathit{rel}}} \\mathit{lam} - 1}$</div>"}︡{"done":true}︡
︠a148df89-f28d-4fe6-ba7a-a2ae71e284aas︠
eq41 = x_np1 == factor((eq40.rhs()/3))
eq41
︡06d36072-da00-4129-b11f-3cc0ba918e02︡{"html":"<div align='center'>$\\displaystyle x_{\\mathit{np}_{1}} = \\frac{F_{R_{\\mathit{rel}}} \\mathit{lam} - 2 \\, \\beta \\mathit{lam} - x_{n}}{F_{R_{\\mathit{rel}}} \\mathit{lam} - 1}$</div>"}︡{"done":true}︡
︠f81b729b-f275-4a4a-aea6-fb33eaa5e3eds︠
eq42 = eq41.substitute(F_R_rel = (1+x_n/3)^(3*3)); eq42
︡2f4be302-4003-4b77-9f66-cbcd30079166︡{"html":"<div align='center'>$\\displaystyle x_{\\mathit{np}_{1}} = \\frac{\\mathit{lam} {\\left(x_{n} + 3\\right)}^{9} - 39366 \\, \\beta \\mathit{lam} - 19683 \\, x_{n}}{\\mathit{lam} {\\left(x_{n} + 3\\right)}^{9} - 19683}$</div>"}︡{"done":true}︡
︠8700a07c-0979-4739-aed7-bfe9dc2d73fes︠
eq42_lin = eq41.substitute(F_R_rel = (1+3*x_n)); eq42_lin
︡d74aa865-dfc4-4f81-ae29-280f4d2baca0︡{"html":"<div align='center'>$\\displaystyle x_{\\mathit{np}_{1}} = -\\frac{2 \\, \\beta \\mathit{lam} - \\mathit{lam} {\\left(3 \\, x_{n} + 1\\right)} + x_{n}}{\\mathit{lam} {\\left(3 \\, x_{n} + 1\\right)} - 1}$</div>"}︡{"done":true}︡
︠a19e6ded-21bd-4862-837e-512c5487c84bs︠
eq42_blim =  eq41.substitute(F_R_rel = (1+x_n)^3,beta=min_symbolic(1,(1-x_n)/(2*lam))); eq42_blim
eq42_blim = eq42_blim.substitute(lam=0.05); eq42_blim
︡534d836d-1724-4efe-8bc2-dda7411224c8︡{"html":"<div align='center'>$\\displaystyle x_{\\mathit{np}_{1}} = \\frac{\\mathit{lam} {\\left(x_{n} + 1\\right)}^{3} - 2 \\, \\mathit{lam} \\min\\left(1, -\\frac{x_{n} - 1}{2 \\, \\mathit{lam}}\\right) - x_{n}}{\\mathit{lam} {\\left(x_{n} + 1\\right)}^{3} - 1}$</div>"}︡{"html":"<div align='center'>$\\displaystyle x_{\\mathit{np}_{1}} = \\frac{0.0500000000000000 \\, {\\left(x_{n} + 1\\right)}^{3} - x_{n} - 0.100000000000000 \\, \\min\\left(1, -10.0000000000000 \\, x_{n} + 10.0000000000000\\right)}{0.0500000000000000 \\, {\\left(x_{n} + 1\\right)}^{3} - 1}$</div>"}︡{"done":true}︡
︠f79689dc-b32f-4425-a6a0-e5e8df123578s︠
eq42 = eq42.substitute(beta=1,lam=0.05)
eq42_lin = eq42_lin.substitute(beta=1,lam=0.05)
eq42
eq42_lin
︡1873be06-b984-4035-8bd8-ada521328df9︡{"html":"<div align='center'>$\\displaystyle x_{\\mathit{np}_{1}} = \\frac{0.0500000000000000 \\, {\\left(x_{n} + 3\\right)}^{9} - 19683 \\, x_{n} - 1968.30000000000}{0.0500000000000000 \\, {\\left(x_{n} + 3\\right)}^{9} - 19683}$</div>"}︡{"html":"<div align='center'>$\\displaystyle x_{\\mathit{np}_{1}} = -\\frac{0.850000000000000 \\, x_{n} + 0.0500000000000000}{0.150000000000000 \\, x_{n} - 0.950000000000000}$</div>"}︡{"done":true}︡
︠63d4de80-ae48-4d03-9a66-d3e1835b4105s︠
# Plot everything together here.  Can plot individual plots below.
x_min=-1
x_max=1
P1 = plot(eq42.rhs(),(x_n,x_min,x_max),legend_label="extrap (1+<x>/3)^(3*3)")
P2 = plot(eq42_lin.rhs(),(x_n,x_min,x_max),color="red",legend_label="linear extrap")
P3 = plot(eq42_blim.rhs(),(x_n,x_min,x_max),color="purple",legend_label="cubic+beta_lim")
P4 = plot(x,(x_min,x_max),color="green",legend_label="fixed point")
P=P1+P2+P3+P4
P.axes_labels(["$<x>_{n}$","$<x>_{n+1}$"])
# \langle and \rangle apparently not implemented in the LaTeX labels
# P.axes_labels(["$\langle x \rangle_{n}$","$\langle x \rangle_{n+1}$"])
show(P)
P.save("x_avg_map.png")
P.save("x_avg_map.pdf")
︡13ff00c0-4695-4b57-8ac2-637bcc25756f︡{"file":{"filename":"/projects/b0fff224-ee44-4488-bd52-7a9b55bcf3db/.sage/temp/compute4-us/31153/tmp_fjvQhz.svg","show":true,"text":null,"uuid":"f0a96c81-4e4f-42e5-a97f-b041c5813c08"},"once":false}︡{"done":true}︡
︠4268e751-d916-4646-8848-34ed09bba134i︠

︡f852ab90-4153-4609-a1ef-a892bf80bcfa︡
︠bafa7e4f-e054-4b5d-a55f-5d11bbe86fc2︠
%md(hide=true)
Conclusions:

This is a plot of $\langle x \rangle$ on the next time step as a function of $\langle x \rangle$ on the previous time step, for lambda = v*dt/dx = 0.05.

The linear extrapolation to the boundary flux has the problem that $\langle x \rangle$ increases every time step and will eventually exceed the bound $\langle x \rangle \le 1$.

Using the cubic extrapolation to the boundary (which is a kind of Pade approximation to the exponential extrapolation), does slow down the rate per step by which $\langle x \rangle$ increases, but it still eventually exceeds the limit.

The only way to rigorously satisfy the limit on $\langle x \rangle$ is to limit beta to be less than 1, where beta is the mulitplier on the interior volume average term $\langle v(x) f(x) \rangle$.  I think the way to interpret this is that when $\langle x_n \rangle$ approaches 1, then the Euler time step is breaking down, because it takes very little time for the whole solultion to advect out of the right boundary.  I think it is okay to apply a limiter there, as if we were reverting to a 2cd order centered method, because the order of accuracy for the time integration only holds in smooth regions, where $|\langle x_n \rangle| \ll 1$, and this limiter on beta preserves an important feature of the solution while retaining at least first-order accuracy in time.


︡204e5456-c273-42c3-9916-f93f92b44c63︡{"done":true,"md":"Conclusions:\n\nThis is a plot of $\\langle x \\rangle$ on the next time step as a function of $\\langle x \\rangle$ on the previous time step, for lambda = v*dt/dx = 0.05.\n\nThe linear extrapolation to the boundary flux has the problem that $\\langle x \\rangle$ increases every time step and will eventually exceed the bound $\\langle x \\rangle \\le 1$.\n\nUsing the cubic extrapolation to the boundary (which is a kind of Pade approximation to the exponential extrapolation), does slow down the rate per step by which $\\langle x \\rangle$ increases, but it still eventually exceeds the limit.\n\nThe only way to rigorously satisfy the limit on $\\langle x \\rangle$ is to limit beta to be less than 1, where beta is the mulitplier on the interior volume average term $\\langle v(x) f(x) \\rangle$.  I think the way to interpret this is that when $\\langle x_n \\rangle$ approaches 1, then the Euler time step is breaking down, because it takes very little time for the whole solultion to advect out of the right boundary.  I think it is okay to apply a limiter there, as if we were reverting to a 2cd order centered method, because the order of accuracy for the time integration only holds in smooth regions, where $|\\langle x_n \\rangle| \\ll 1$, and this limiter on beta preserves an important feature of the solution while retaining at least first-order accuracy in time."}
︠efb1ae29-6c2a-4161-9bc2-2f1e250d6582s︠
%var x, y, mx, my, mxy
e100 = integrate((1+mx*x+mx*y+mxy*x*y)^3*(1+mxy*x*y)^6,x,-1,1)
︡1c96059d-c5fc-422f-bed3-f131ef160b0b︡{"done":true}︡
︠76419b4c-0f16-4adc-acb8-0fa031202ffcs︠
e101 = simplify(e100)
e101
︡569b24cb-1e4a-43ed-9cc8-e77ddfd13049︡{"html":"<div align='center'>$\\displaystyle \\frac{1}{840} \\, {\\left(120 \\, \\mathit{mx}^{3} \\mathit{mxy}^{6} + 315 \\, \\mathit{mx}^{2} \\mathit{mxy}^{7} + 280 \\, \\mathit{mx} \\mathit{mxy}^{8} + 84 \\, \\mathit{mxy}^{9}\\right)} y^{9} + \\frac{1}{840} \\, {\\left(120 \\, \\mathit{mx}^{3} \\mathit{mxy}^{6} - 315 \\, \\mathit{mx}^{2} \\mathit{mxy}^{7} + 280 \\, \\mathit{mx} \\mathit{mxy}^{8} - 84 \\, \\mathit{mxy}^{9}\\right)} y^{9} + \\frac{1}{120} \\, {\\left(12 \\, {\\left(3 \\, \\mathit{mx} + 10\\right)} \\mathit{mxy}^{8} + 120 \\, \\mathit{mx}^{3} \\mathit{mxy}^{5} + 40 \\, {\\left(2 \\, \\mathit{mx}^{2} + 9 \\, \\mathit{mx}\\right)} \\mathit{mxy}^{7} + 45 \\, {\\left(\\mathit{mx}^{3} + 8 \\, \\mathit{mx}^{2}\\right)} \\mathit{mxy}^{6}\\right)} y^{8} - \\frac{1}{120} \\, {\\left(12 \\, {\\left(3 \\, \\mathit{mx} - 10\\right)} \\mathit{mxy}^{8} + 120 \\, \\mathit{mx}^{3} \\mathit{mxy}^{5} - 40 \\, {\\left(2 \\, \\mathit{mx}^{2} - 9 \\, \\mathit{mx}\\right)} \\mathit{mxy}^{7} + 45 \\, {\\left(\\mathit{mx}^{3} - 8 \\, \\mathit{mx}^{2}\\right)} \\mathit{mxy}^{6}\\right)} y^{8} + \\frac{1}{420} \\, {\\left(14 \\, {\\left(9 \\, \\mathit{mx}^{2} + 80 \\, \\mathit{mx} + 135\\right)} \\mathit{mxy}^{7} + 1260 \\, \\mathit{mx}^{3} \\mathit{mxy}^{4} + 35 \\, {\\left(4 \\, \\mathit{mx}^{3} + 63 \\, \\mathit{mx}^{2} + 144 \\, \\mathit{mx}\\right)} \\mathit{mxy}^{6} + 90 \\, {\\left(12 \\, \\mathit{mx}^{3} + 49 \\, \\mathit{mx}^{2}\\right)} \\mathit{mxy}^{5}\\right)} y^{7} - \\frac{1}{420} \\, {\\left(14 \\, {\\left(9 \\, \\mathit{mx}^{2} - 80 \\, \\mathit{mx} + 135\\right)} \\mathit{mxy}^{7} - 1260 \\, \\mathit{mx}^{3} \\mathit{mxy}^{4} - 35 \\, {\\left(4 \\, \\mathit{mx}^{3} - 63 \\, \\mathit{mx}^{2} + 144 \\, \\mathit{mx}\\right)} \\mathit{mxy}^{6} - 90 \\, {\\left(12 \\, \\mathit{mx}^{3} - 49 \\, \\mathit{mx}^{2}\\right)} \\mathit{mxy}^{5}\\right)} y^{7} + \\frac{1}{60} \\, {\\left(2 \\, {\\left(3 \\, \\mathit{mx}^{3} + 70 \\, \\mathit{mx}^{2} + 315 \\, \\mathit{mx} + 360\\right)} \\mathit{mxy}^{6} + 300 \\, \\mathit{mx}^{3} \\mathit{mxy}^{3} + 15 \\, {\\left(9 \\, \\mathit{mx}^{3} + 72 \\, \\mathit{mx}^{2} + 112 \\, \\mathit{mx}\\right)} \\mathit{mxy}^{5} + 90 \\, {\\left(5 \\, \\mathit{mx}^{3} + 14 \\, \\mathit{mx}^{2}\\right)} \\mathit{mxy}^{4}\\right)} y^{6} - \\frac{1}{60} \\, {\\left(2 \\, {\\left(3 \\, \\mathit{mx}^{3} - 70 \\, \\mathit{mx}^{2} + 315 \\, \\mathit{mx} - 360\\right)} \\mathit{mxy}^{6} + 300 \\, \\mathit{mx}^{3} \\mathit{mxy}^{3} + 15 \\, {\\left(9 \\, \\mathit{mx}^{3} - 72 \\, \\mathit{mx}^{2} + 112 \\, \\mathit{mx}\\right)} \\mathit{mxy}^{5} + 90 \\, {\\left(5 \\, \\mathit{mx}^{3} - 14 \\, \\mathit{mx}^{2}\\right)} \\mathit{mxy}^{4}\\right)} y^{6} + \\frac{1}{168} \\, {\\left(7 \\, {\\left(16 \\, \\mathit{mx}^{3} + 189 \\, \\mathit{mx}^{2} + 576 \\, \\mathit{mx} + 504\\right)} \\mathit{mxy}^{5} + 840 \\, \\mathit{mx}^{3} \\mathit{mxy}^{2} + 24 \\, {\\left(45 \\, \\mathit{mx}^{3} + 245 \\, \\mathit{mx}^{2} + 294 \\, \\mathit{mx}\\right)} \\mathit{mxy}^{4} + 126 \\, {\\left(16 \\, \\mathit{mx}^{3} + 35 \\, \\mathit{mx}^{2}\\right)} \\mathit{mxy}^{3}\\right)} y^{5} + \\frac{1}{168} \\, {\\left(7 \\, {\\left(16 \\, \\mathit{mx}^{3} - 189 \\, \\mathit{mx}^{2} + 576 \\, \\mathit{mx} - 504\\right)} \\mathit{mxy}^{5} + 840 \\, \\mathit{mx}^{3} \\mathit{mxy}^{2} + 24 \\, {\\left(45 \\, \\mathit{mx}^{3} - 245 \\, \\mathit{mx}^{2} + 294 \\, \\mathit{mx}\\right)} \\mathit{mxy}^{4} + 126 \\, {\\left(16 \\, \\mathit{mx}^{3} - 35 \\, \\mathit{mx}^{2}\\right)} \\mathit{mxy}^{3}\\right)} y^{5} + \\frac{1}{40} \\, {\\left({\\left(75 \\, \\mathit{mx}^{3} + 600 \\, \\mathit{mx}^{2} + 1400 \\, \\mathit{mx} + 1008\\right)} \\mathit{mxy}^{4} + 120 \\, \\mathit{mx}^{3} \\mathit{mxy} + 80 \\, {\\left(5 \\, \\mathit{mx}^{3} + 21 \\, \\mathit{mx}^{2} + 21 \\, \\mathit{mx}\\right)} \\mathit{mxy}^{3} + 30 \\, {\\left(15 \\, \\mathit{mx}^{3} + 28 \\, \\mathit{mx}^{2}\\right)} \\mathit{mxy}^{2}\\right)} y^{4} - \\frac{1}{40} \\, {\\left({\\left(75 \\, \\mathit{mx}^{3} - 600 \\, \\mathit{mx}^{2} + 1400 \\, \\mathit{mx} - 1008\\right)} \\mathit{mxy}^{4} + 120 \\, \\mathit{mx}^{3} \\mathit{mxy} + 80 \\, {\\left(5 \\, \\mathit{mx}^{3} - 21 \\, \\mathit{mx}^{2} + 21 \\, \\mathit{mx}\\right)} \\mathit{mxy}^{3} + 30 \\, {\\left(15 \\, \\mathit{mx}^{3} - 28 \\, \\mathit{mx}^{2}\\right)} \\mathit{mxy}^{2}\\right)} y^{4} + \\frac{1}{70} \\, {\\left({\\left(200 \\, \\mathit{mx}^{3} + 1225 \\, \\mathit{mx}^{2} + 2352 \\, \\mathit{mx} + 1470\\right)} \\mathit{mxy}^{3} + 70 \\, \\mathit{mx}^{3} + 35 \\, {\\left(18 \\, \\mathit{mx}^{3} + 63 \\, \\mathit{mx}^{2} + 56 \\, \\mathit{mx}\\right)} \\mathit{mxy}^{2} + 105 \\, {\\left(4 \\, \\mathit{mx}^{3} + 7 \\, \\mathit{mx}^{2}\\right)} \\mathit{mxy}\\right)} y^{3} + \\frac{1}{70} \\, {\\left({\\left(200 \\, \\mathit{mx}^{3} - 1225 \\, \\mathit{mx}^{2} + 2352 \\, \\mathit{mx} - 1470\\right)} \\mathit{mxy}^{3} + 70 \\, \\mathit{mx}^{3} + 35 \\, {\\left(18 \\, \\mathit{mx}^{3} - 63 \\, \\mathit{mx}^{2} + 56 \\, \\mathit{mx}\\right)} \\mathit{mxy}^{2} + 105 \\, {\\left(4 \\, \\mathit{mx}^{3} - 7 \\, \\mathit{mx}^{2}\\right)} \\mathit{mxy}\\right)} y^{3} + \\frac{1}{10} \\, {\\left(15 \\, \\mathit{mx}^{3} + {\\left(25 \\, \\mathit{mx}^{3} + 126 \\, \\mathit{mx}^{2} + 210 \\, \\mathit{mx} + 120\\right)} \\mathit{mxy}^{2} + 30 \\, \\mathit{mx}^{2} + 5 \\, {\\left(9 \\, \\mathit{mx}^{3} + 28 \\, \\mathit{mx}^{2} + 24 \\, \\mathit{mx}\\right)} \\mathit{mxy}\\right)} y^{2} - \\frac{1}{10} \\, {\\left(15 \\, \\mathit{mx}^{3} + {\\left(25 \\, \\mathit{mx}^{3} - 126 \\, \\mathit{mx}^{2} + 210 \\, \\mathit{mx} - 120\\right)} \\mathit{mxy}^{2} - 30 \\, \\mathit{mx}^{2} + 5 \\, {\\left(9 \\, \\mathit{mx}^{3} - 28 \\, \\mathit{mx}^{2} + 24 \\, \\mathit{mx}\\right)} \\mathit{mxy}\\right)} y^{2} + 2 \\, \\mathit{mx}^{2} + \\frac{1}{20} \\, {\\left(20 \\, \\mathit{mx}^{3} + 60 \\, \\mathit{mx}^{2} + {\\left(24 \\, \\mathit{mx}^{3} + 105 \\, \\mathit{mx}^{2} + 160 \\, \\mathit{mx} + 90\\right)} \\mathit{mxy} + 60 \\, \\mathit{mx}\\right)} y + \\frac{1}{20} \\, {\\left(20 \\, \\mathit{mx}^{3} - 60 \\, \\mathit{mx}^{2} + {\\left(24 \\, \\mathit{mx}^{3} - 105 \\, \\mathit{mx}^{2} + 160 \\, \\mathit{mx} - 90\\right)} \\mathit{mxy} + 60 \\, \\mathit{mx}\\right)} y + 2$</div>"}︡{"done":true}︡
︠7ed7467e-bcd8-4c9b-adb2-648aae2fd1ees︠
e102 = integrate(e101,y,-1,1)/4
e102
︡1060f247-8bce-4563-bb55-7b58366b988f︡{"html":"<div align='center'>$\\displaystyle \\frac{1}{180} \\, {\\left(3 \\, \\mathit{mx} + 10\\right)} \\mathit{mxy}^{8} - \\frac{1}{180} \\, {\\left(3 \\, \\mathit{mx} - 10\\right)} \\mathit{mxy}^{8} + \\frac{1}{54} \\, {\\left(2 \\, \\mathit{mx}^{2} + 9 \\, \\mathit{mx}\\right)} \\mathit{mxy}^{7} + \\frac{1}{54} \\, {\\left(2 \\, \\mathit{mx}^{2} - 9 \\, \\mathit{mx}\\right)} \\mathit{mxy}^{7} + \\frac{1}{1680} \\, {\\left(47 \\, \\mathit{mx}^{3} + 560 \\, \\mathit{mx}^{2} + 1260 \\, \\mathit{mx} + 1440\\right)} \\mathit{mxy}^{6} - \\frac{1}{1680} \\, {\\left(47 \\, \\mathit{mx}^{3} - 560 \\, \\mathit{mx}^{2} + 1260 \\, \\mathit{mx} - 1440\\right)} \\mathit{mxy}^{6} + \\frac{1}{504} \\, {\\left(109 \\, \\mathit{mx}^{3} + 648 \\, \\mathit{mx}^{2} + 1008 \\, \\mathit{mx}\\right)} \\mathit{mxy}^{5} - \\frac{1}{504} \\, {\\left(109 \\, \\mathit{mx}^{3} - 648 \\, \\mathit{mx}^{2} + 1008 \\, \\mathit{mx}\\right)} \\mathit{mxy}^{5} + \\frac{1}{2800} \\, {\\left(2025 \\, \\mathit{mx}^{3} + 8400 \\, \\mathit{mx}^{2} + 9800 \\, \\mathit{mx} + 7056\\right)} \\mathit{mxy}^{4} - \\frac{1}{2800} \\, {\\left(2025 \\, \\mathit{mx}^{3} - 8400 \\, \\mathit{mx}^{2} + 9800 \\, \\mathit{mx} - 7056\\right)} \\mathit{mxy}^{4} + \\frac{1}{70} \\, {\\left(95 \\, \\mathit{mx}^{3} + 294 \\, \\mathit{mx}^{2} + 294 \\, \\mathit{mx}\\right)} \\mathit{mxy}^{3} - \\frac{1}{70} \\, {\\left(95 \\, \\mathit{mx}^{3} - 294 \\, \\mathit{mx}^{2} + 294 \\, \\mathit{mx}\\right)} \\mathit{mxy}^{3} + \\frac{1}{120} \\, {\\left(185 \\, \\mathit{mx}^{3} + 504 \\, \\mathit{mx}^{2} + 420 \\, \\mathit{mx} + 240\\right)} \\mathit{mxy}^{2} - \\frac{1}{120} \\, {\\left(185 \\, \\mathit{mx}^{3} - 504 \\, \\mathit{mx}^{2} + 420 \\, \\mathit{mx} - 240\\right)} \\mathit{mxy}^{2} + 2 \\, \\mathit{mx}^{2} + \\frac{1}{60} \\, {\\left(63 \\, \\mathit{mx}^{3} + 140 \\, \\mathit{mx}^{2} + 120 \\, \\mathit{mx}\\right)} \\mathit{mxy} - \\frac{1}{60} \\, {\\left(63 \\, \\mathit{mx}^{3} - 140 \\, \\mathit{mx}^{2} + 120 \\, \\mathit{mx}\\right)} \\mathit{mxy} + 1$</div>"}︡{"done":true}︡
︠3e0c46ea-4f38-4c0e-92e8-18c673b6bb73s︠
e103 = simplify(e102)
︡fadeb9c3-b703-4549-b340-9e9d3bef86db︡{"done":true}︡
︠1f7b8487-3163-481d-b358-b407b50f8a35s︠
e104 = e103.substitute(mx=1,my=1,mxy=1)
︡22c2399b-7d5a-4192-a621-468c4bf3616f︡{"done":true}︡
︠02ec034c-14ee-4fe9-a582-032503f0e3b3s︠
float(e104)
︡ca1bdb04-495f-444c-8344-4c6325520557︡{"html":"<div align='center'>$\\displaystyle 44.6442328042$</div>"}︡{"done":true}︡
︠fd910d95-f298-4875-97aa-7d81aedf80cfs︠
2048/float(104)
︡1de9cc33-c8ae-4a9f-820f-489e2788e1ef︡{"html":"<div align='center'>$\\displaystyle 19.6923076923$</div>"}︡{"done":true}︡
︠a324e900-d44e-4888-a6a8-bceef2f117f6s︠
integrate((1+mx*x)^3,x,-1,1)
︡d9ea11ed-22cd-481b-a71b-bca09aad7a5e︡{"html":"<div align='center'>$\\displaystyle 2 \\, \\mathit{mx}^{2} + 2$</div>"}︡{"done":true}︡
︠f856d337-d19b-4874-98c2-c2d4f6d7c3d1︠









